% Solve the model in "Fundamentals News, Global Liquidity and Macroprudential Policy" 
% The code presented here does not use any computational trick for pedagogical purpose

clear all;
clc;
close all

%% 1. Parameter Values

%%%%%%%%%%%%%%%%%  Model Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global omega sigma ita beta r Zprob  B  bp  c uptd Prob Q
sigma=2; 
ita=(1/0.83)-1;
beta=0.91;
omega=0.32;
kappa=0.32;
rho=0.54; 
sigmay=0.059; 
NS=3;
NSS=NS*NS;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%  Technical parameters  %%%%%%%%%%%%%%%%%%%%%%%%%
uptd=0.2; %Change to 0.1 if policy function fluctuate
outfreq=20; %Display frequency
iter_tol=600; % Iteration tolerance
tol=1e-5; %Numerical tolerance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%% Incorporate news shocks  %%%%%%%%%%%%%%%%%%%%
info=2/3; % =1 is perfect signal; =1/NS is useless signal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%% IR shocks %%%%%%%%%%%%%%%%
Rlow=-0.03283215044;
Rhigh=0.014500;
Fll=0.6;
Fhh=14/15;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%% Grid assignment %%%%%%%%%%%%%%%%
NB=100; %number of nodes.
bmin=-1.11; 
bmax=-0.31;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% 2. Construction of Markov Chain

%%%%%%%%%%%%%%%%%%%%%%  Tauchen Hussey   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
[Z,Zprob] = tauchenhussey(NS,0,rho,sigmay,sigmay); % Row are today, column are tomorrow
Z=exp(Z);
ZZ=diag(Z);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%  Incorporate News Structure %%%%%%%%%%%%%%%%%%%%%%%
Psz=zeros(NS); %transitional probability of s_t given z_{t+1}
Psz=Psz+(1-info)/(NS-1).*(1-eye(NS))+eye(NS).*info;

Pzsz=zeros(NSS,NS); % P(z'|s,z)  Equation (2) in Durdu et al (2013)
for i=1:NS % s
    for j=1:NS % z
        for l=1:NS % z'
            k=(i-1)*NS+j;
            Pzsz(k,l)=(Psz(l,i)*Zprob(j,l))/(Zprob(j,:)*Psz(:,i));
        end
    end
end
%Order: z1'|s1z1; z1'|s1z2; z1'|s2z1; z1'|s2z2; z2'|s1z1......z2'|s2z2;

NProb=zeros(NSS,NSS);  % Equation (3) in Durdu et al (2013)  z's'|zs
for i=1:NS % s
    for j=1:NS % z
        for l=1:NS % z'
            for k=1:NS %s'
                ii=(i-1)*NS+j; %today
                jj=(k-1)*NS+l; %tomorrow
                NProb(ii,jj)=Pzsz(ii,l)*((Zprob(l,:)*Psz(:,k)));
            end
        end
    end
end
%Order: s1z1, s1z2, s2z1, s2z2

%%%%%%%%%%%%%%%%%%%%%% Interest Rate Regime  %%%%%%%%%%%%%%%%%%%%%%%%%
r=[Rhigh,Rlow];
Rprob=[Fhh,1-Fhh;1-Fll,Fll]; % Persistent
Prob=[NProb*Rprob(1,1),NProb*Rprob(1,2);NProb*Rprob(2,1),NProb*Rprob(2,2)]; %Today is goods shock, bad to good and interest rate shock bad to good; 
State_Index=[1:length(Prob)]';


B=zeros(NB,1);
B(1)=bmin;
%%%%%%%%%%%%%%% Symmetric  %%%%%%%%%%%%%%%%%%%%%
for i=2:NB;
    B(i)=B(i-1)+(bmax-B(i-1))/((NB-i+1)^1.05); 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Ntotal=NSS*length(r);
yt=repmat(Z',NB,1);
yt=repmat(yt,1,NS*length(r));

R=[repmat(r(1),NB,NSS),repmat(r(2),NB,NSS)]; % IR spaces

Q=1./(1+R);

b=repmat(B,1,Ntotal);
yn=ones(NB,Ntotal);
price=ones(NB,Ntotal);

bp=repmat(B,1,Ntotal); 
c=ones(NB,Ntotal);
EE=zeros(NB,Ntotal);
cbind=ones(NB,Ntotal);
bpmax=-kappa*(price+yt);
mup=ones(NB,Ntotal);
emu=zeros(NB,Ntotal);

cSP=ones(NB,Ntotal);
priceSP=ones(NB,Ntotal);
bpSP=ones(NB,Ntotal);
cbindSP=ones(NB,Ntotal);
bpmaxSP=ones(NB,Ntotal);
mupSP=ones(NB,Ntotal);
EESP=zeros(NB,Ntotal);
emuSP=zeros(NB,Ntotal);
cbind=c;

%% 3. Decentralized Equilibrium
iter=0;
d2=100;
options = optimoptions('fsolve','Display','off');
disp('DE Iter      Norm');
while d2>tol && iter <iter_tol    
    
oldp=price;
oldc=c;
oldbp=bp;

totalc=(omega*c.^(-ita)+(1-omega)*yn.^(-ita)); 
mup=real(omega*totalc.^(sigma/ita).*(totalc.^(-1/ita-1)).*(c.^(-ita-1)));  %mup(b,y)

for i=1:NB
    for j=1:Ntotal
        emu(i,j)=beta*(1+R(i,j))*interp1(B,mup,bp(i,j),'linear','extrap')*Prob(j,:)'; %EMU is expected marginal utility tomorrow in today's grid. 
    end
end

for i=1:NB
    for j=1:Ntotal
        EE(i,j)=(omega*cbind(i,j)^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cbind(i,j)^(-ita-1)-emu(i,j); 
        if EE(i,j)>1e-5
            bp(i,j)=bpmax(i,j);
            c(i,j)=cbind(i,j);
        else
%             f = @(cc) abs((omega*cc^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cc^(-ita-1)-emu(i,j));  %Use minimization routine is faster
%             [c(i,j),EE(i,j)]=fminbnd(f,0.001,2); 
            f = @(cc) (omega*cc^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cc^(-ita-1)-emu(i,j);           
            [c(i,j),EE(i,j)]=fsolve(f,0.001,options);             
            bp(i,j)=max((1/Q(i,j))*(yt(i,j)+b(i,j)-c(i,j)),bmin);
            bp(i,j)=min(bp(i,j),bmax);             
        end
    end
end
price=real((1-omega)/omega*c.^(1+ita));
%===============Check collateral constraint==============================
c=b+yt-max(Q.*bp,-kappa*(price+yt));  %qb'>=-kappa*(yt+yn*pn)
price=real((1-omega)/omega*c.^(1+ita));
bp=(1./Q).*(b+yt-c);
%bp=min(bp,bmax);
%========================================================================

%=====================Updating rule.Must be slow, important!==============
bp=uptd*bp+(1-uptd)*oldbp;
c=uptd*c+(1-uptd)*oldc;
price=uptd*price+(1-uptd)*oldp;
%========================================================================

bpmax=-(1./Q).*kappa.*(price+yt);
bpmax(bpmax>bmax)=bmax;
bpmax(bpmax<bmin)=bmin;
cbind=b+yt-Q.*bpmax;

iter=iter+1;

d2=max([max(max(abs(c-oldc))),max(max(abs(bp-oldbp)))]);

if mod(iter, outfreq) == 0;
    fprintf('%d          %1.7f \n',iter,d2);
end
end

%% 4. Social Planner

uptd=uptd/5;
cSP=c;
priceSP=price;
bpSP=bp;
cbindSP=c;
bpmaxSP=bpSP;
EESP=EE;

d2=100;
iter=0;
disp('SPP Iter     Norm');

while d2>tol && iter <iter_tol*2 
    
oldpSP=priceSP;
oldcSP=cSP;
oldbpSP=bpSP;

totalcSP=(omega*cSP.^(-ita)+(1-omega)*yn.^(-ita)); 
mupSP=real(omega*totalcSP.^(sigma/ita).*(totalcSP.^(-1/ita-1)).*(cSP.^(-ita-1)));  %mup(b,y)

psi=(1-omega)/omega*(ita+1)*(cSP./yn).^ita*kappa;
mupSP=EESP.*psi+mupSP;

for i=1:NB
    for j=1:Ntotal
        emuSP(i,j)=beta*(1+R(i,j))*interp1(B,mupSP,bpSP(i,j),'linear','extrap')*Prob(j,:)'; %EMU is expected marginal utility tomorrow in today's grid. 
    end
end

for i=1:NB
    for j=1:Ntotal
        EESP(i,j)=(omega*cbindSP(i,j)^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cbindSP(i,j)^(-ita-1)-emuSP(i,j); 
        if EESP(i,j)>1e-5
            bpSP(i,j)=bpmaxSP(i,j);
            cSP(i,j)=cbindSP(i,j);
        else
%             f = @(cc) abs((omega*cc^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cc^(-ita-1)-emuSP(i,j));
%             [cSP(i,j),EESP(i,j)]=fminbnd(f,0.001,2);
            f = @(cc) (omega*cc^(-ita)+1-omega)^(sigma/ita-1/ita-1)*omega*cc^(-ita-1)-emuSP(i,j);
            [cSP(i,j),EESP(i,j)]=fsolve(f,0.001,options);               
            bpSP(i,j)=max((1/Q(i,j))*(yt(i,j)+b(i,j)-cSP(i,j)),bmin);
            bpSP(i,j)=min(bpSP(i,j),bmax);
        end
    end
end
priceSP=real((1-omega)/omega*cSP.^(1+ita));
%===============Check collateral constraint==============================
cSP=b+yt-max(Q.*bpSP,-kappa*(priceSP+yt));
priceSP=real((1-omega)/omega*cSP.^(1+ita));
bpSP=(1./Q).*(b+yt-cSP);
%bp=max(bp,min);
%========================================================================

%=====================Updating rule.Must be slow, important!==============
bpSP=uptd*bpSP+(1-uptd)*oldbpSP;
cSP=uptd*cSP+(1-uptd)*oldcSP;
priceSP=uptd*priceSP+(1-uptd)*oldpSP;
%========================================================================

bpmaxSP=-(1./Q).*kappa.*(priceSP+yt);
bpmaxSP(bpmaxSP>bmax)=bmax;
bpmaxSP(bpmaxSP<bmin)=bmin;
cbindSP=b+yt-Q.*bpmaxSP;

d2=max([max(max(abs(cSP-oldcSP))),max(max(abs(bpSP-oldbpSP)))]);

iter=iter+1;
if mod(iter, outfreq*5) == 0;
    fprintf('%d          %1.7f \n',iter,d2);
end
end

%% 5. Welfare Calculation
V=zeros(NB,Ntotal);
VSP=zeros(NB,Ntotal);

U=(totalc.^(-1/ita)).^(1-sigma)./(1-sigma);
USP=(totalcSP.^(-1/ita)).^(1-sigma)./(1-sigma);

EV=U;
EVSP=USP;

d3=100;
iter=0;
disp('Value Iter     Norm');
while d3>tol && iter <iter_tol
for i=1:NB
    for j=1:Ntotal
        EV(i,j)=beta*interp1(B,V,bp(i,j),'linear','extrap')*Prob(j,:)'; %EV is expected value tomorrow in today's grid.
        EVSP(i,j)=beta*interp1(B,VSP,bpSP(i,j),'linear','extrap')*Prob(j,:)';          
    end
end

V_new=U+EV;
VSP_new=USP+EVSP;

d3=max([max(max(abs((V_new-V)./V))),max(max(abs((VSP_new-VSP)./VSP)))]);

iter=iter+1;

V=V_new;
VSP=VSP_new;
if mod(iter, outfreq) == 0;
    fprintf('%d          %1.7f \n',iter,d3);
end

end
Wf=(VSP./V).^(1/(1-sigma))-1;

%% 6. Optimal Tax
mupSP=mupSP-EESP.*psi;
for i=1:NB
    for j=1:Ntotal
        emuSP(i,j)=beta*(1+R(i,j))*interp1(B,mupSP,bpSP(i,j),'linear','extrap')*Prob(j,:)'; 
    end
end
Tao=mupSP./emuSP;

% Tax is set to one when constraint binds
AN=bpSP(2:end,:)-bpSP(1:end-1,:);

for i=1:Ntotal
    IN(i)=find(AN(:,i)>0,1,'first');
end


for j=1:Ntotal
    if IN(j)>1
        Tao(1:IN(j)-1,j)=1;
    end
end

Tao(Tao<1)=1;
